package astAlgos;

import ast.*;

/**
 * This AST visitor algorithm computes the integer evaluation of the AST.
 * It assumes that all sub-trees evaluate to Integer objects.
 * The return value from executing this algorithm is an Integer object.
 * This algorithm defines the symantics of the leaf and binary operations of
 * the AST.
 * <pre>
 * Example:
 * new Add(new IntLeaf(2), new Multiply(new IntLeaf(5), new IntLeaf(7)))
 *     +
 *   /   \
 *  2     *
 *      /   \
 *     5     7
 * </pre>
 * evaluates to: new Integer(37
)
 */
public class IntEvaluator implements IASTAlgo {
    public final static IntEvaluator Singleton = new IntEvaluator();
    private IntEvaluator() {
    }

    /**
     * Returns the value held by a leaf object.  The value is assumed to be an
     * Integer object.
     * @param host The ILeaf object that is executing this algorithm.
     * @param param Not used.
     * @return The Integer value of the leaf object.
     */
    public Object intLeafCase(ILeaf host, Object param) {
        return host.getValue();
    }

    /**
     * Returns the value associated with tje leaf object.
     * The leaf object's value is treated as an id ("key") and the actual value
     * associated
     * with that id is retrieved from the input parameter, which is an
     * IEnvironment.
     * The value is assumed to be an Integer object.
     * @param host The ILeaf object that is executing this algorithm.
     * @param param An IEnvironment object.
     * @return The value the environment associates with the ILeaf's value (id).
     */
    public Object varLeafCase(ILeaf host, Object param) {
        return ((IEnvironment) param).getValue(host.getValue());
    }

    /**
     * Returns the integer sum of the integer evaluation of the left-hand and
     * right-hand AST's of a binary operation (IBinOp) object.
     * The left-hand tree is recursively evaluated, and then the right-hand
     * tree is recursively evaluated.
     * The two results are added together and returned as an Integer object.
     * @param host The IBinOp object that is executing this algorithm.
     * @param param Not used.
     * @return The sum of the lhs and rhs AST's as an Integer object.
     */
    public Object addCase(IBinOp host, Object param) {
        return new Integer(
                    ((Integer)(host.getLHS().execute(this, param))).intValue() +
                    ((Integer)(host.getRHS().execute(this, param))).intValue());
    }

    /**
     * Returns the integer product of the integer evaluation of the left-hand
     * and right-hand AST's of a binary operation (IBinOp) object.
     * The left-hand tree is recursively evaluated, and then the right-hand tree
     * is recursively evaluated.
     * The two results are multiplied together and returned as an Integer object.
     * @param host The IBinOp object that is executing this algorithm.
     * @param param Not used.
     * @return The product of the lhs and rhs AST's as an Integer object.
     */
    public Object multiplyCase(IBinOp host, Object param) {
        return new Integer(
                    ((Integer)(host.getLHS().execute(this, param))).intValue() *
                    ((Integer)(host.getRHS().execute(this, param))).intValue());
    }
}

